<?php
defined( 'WPINC' ) or die;

class Voice_Forms_Plugin extends WP_Stack_Plugin2 {

    /**
     * @var self
     */
    public static $plugin_directory_path = null;
    public static $vf_ios = false;
    public static $vf_url = "";
    public static $is_chrome = false;
    public static $vf_license_key = "";
    public static $vf_api_access_key = null;
    public static $vf_admin_notice_logo = "";
    public static $vf_selected_language = "English";
    public static $vf_file_type  = '.min';
    //public static $vf_file_type = ''; // For debugging

    /**
     * Plugin version.
     */
    const VERSION = '1.1.0';
    
    /**
     * Constructs the object, hooks in to `plugins_loaded`.
     */
    protected function __construct()
    {
        // Get database values
        self::$vf_license_key = get_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['license_key'], null);
        self::$vf_license_key = self::vf_sanitize_variable_for_local_script(self::$vf_license_key);

        // Get API access key.
        self::$vf_api_access_key = get_option('vf_api_system_key', null);
        self::$vf_api_access_key = self::vf_sanitize_variable_for_local_script(self::$vf_api_access_key);

        self::$vf_selected_language = get_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['selected_language'], null);
        self::$vf_selected_language = self::vf_sanitize_variable_for_local_script(self::$vf_selected_language);

        // Detect OS by user agent
        $iPod   = stripos($_SERVER['HTTP_USER_AGENT'],"iPod");
        $iPhone = stripos($_SERVER['HTTP_USER_AGENT'],"iPhone");
        $iPad   = stripos($_SERVER['HTTP_USER_AGENT'],"iPad");
        $chrome_browser = stripos($_SERVER['HTTP_USER_AGENT'],"Chrome");

        if (!($iPod == false && $iPhone == false && $iPad == false)) { /*self::$vf_ios = true;*/ }

        if ($chrome_browser != false) { self::$is_chrome = true; }

        $this->hook( 'plugins_loaded', 'add_hooks' );
    }

    /**
     * Adds hooks.
     */
    public function add_hooks()
    {
        $this->hook( 'init' );
        $this->hook( 'admin_enqueue_scripts', 'enqueue_admin_scripts' );

        if ( self::$is_chrome == true 
            || (self::$is_chrome == false && !empty(self::$vf_license_key) && !empty(self::$vf_api_access_key)) ) {
            $this->hook( 'wp_enqueue_scripts', 'enqueue_frontend_scripts' );
        }

        // Register the STT service call action
        add_action ( 'wp_ajax_' . 'vf_log_service_call', array($this, 'vf_log_service_call'));
        add_action ( 'wp_ajax_nopriv_' . 'vf_log_service_call', array($this, 'vf_log_service_call'));

        // Register action to hook into admin_notices to display dahsboard notices when license key is missing or invalid
        if (empty(self::$vf_license_key) || empty(self::$vf_api_access_key)) {
            add_action( 'admin_notices', array($this, 'go_pro_notice'));            
        } 
    }

    /**
     * Method as action to invoke when license key is missing
     */
    public function go_pro_notice() {
    ?>
        <div class="notice notice-warning is-dismissible">
            <p> <?php echo self::$vf_admin_notice_logo; ?>
                <br/>
                <a target="blank" href="https://speak2web.com/plugin/#plan"><?php echo VF_LANGUAGE_LIBRARY['other']['goProNotice']['goPro']; ?> </a> <?php echo VF_LANGUAGE_LIBRARY['other']['goProNotice']['supportMoreBrowsers']; ?> 
            </p>
        </div>
    <?php
    }


    /**
     * Initializes the plugin, registers textdomain, etc.
     * Most of WP is loaded at this stage, and the user is authenticated
     */
    public function init()
    {
        self::$vf_url = $this->get_url();
        self::$vf_admin_notice_logo = "<img style='margin-left: -7px;vertical-align:middle;width:110px; height: 36px;' src='".self::$vf_url."images/speak2web_logo.png'/>|<b> Voice Forms</b>";

        // Get plugin directory path and add trailing slash if needed (For browser compatibility)
        self::$plugin_directory_path = plugin_dir_path(__DIR__);
        $trailing_slash = substr(self::$plugin_directory_path, -1);

        if ($trailing_slash != '/') { self::$plugin_directory_path .= '/'; }

        if ( isset($GLOBALS['pagenow']) && $GLOBALS['pagenow'] == 'plugins.php' ) {
            add_filter( 'plugin_row_meta', array(&$this, 'custom_plugin_row_meta'), 10, 2);
        }

        $this->load_textdomain( 'voice-forms', '/languages' );

        // To enable floating mic by default (only when 'vf_floating_mic' option is missing from DB)
        $is_vf_floating_mic_exist = get_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['floating_mic']);

        if ($is_vf_floating_mic_exist === false) {
            update_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['floating_mic'], 'yes');
        }
    }

    /**
     * Class method to broadcast 'settings' page data in 'vf.speech-handler.js'
     */
    public static function broadcast_api_meta_data()
    {
        $stt_model = '&model=en-US_BroadbandModel';

        switch (strtolower(self::$vf_selected_language)) {
            case 'german':
                $stt_model = '&model=de-DE_BroadbandModel';
                break;
            case 'portuguese':
                $stt_model = '&model=pt-BR_BroadbandModel';
                break;
            case 'chinese':
                $stt_model = '&model=zh-CN_BroadbandModel';
                break;
            case 'french':
                $stt_model = '&model=fr-FR_BroadbandModel';
                break;
            case 'japanese':
                $stt_model = '&model=ja-JP_BroadbandModel';
                break;
            case 'korean':
                $stt_model = '&model=ko-KR_BroadbandModel';
                break;
            case 'spanish':
                $stt_model = '&model=es-ES_BroadbandModel';
                break;
            default:
               $stt_model = '&model=en-US_BroadbandModel';
        }

        $web_socket_url = array(
            'url'     => 'wss://stream.watsonplatform.net/speech-to-text/api/v1/recognize',
            'tokenQs' => '?access_token=',
            'otherQs' => $stt_model
        );
        $token_api_url = 'https://yjonpgjqs9.execute-api.us-east-1.amazonaws.com/V2';

        // Make web socket url and token url available to 'vf.speech-handler.js'
        wp_localize_script( 'vf.speech-handler', 'vfWebSocketUrl', $web_socket_url);
        wp_localize_script( 'vf.speech-handler', 'vfTokenApiUrl', $token_api_url);

        // Make token url available to 'vf.audio-input-handler.js'
        wp_localize_script( 'vf.audio-input-handler', 'vfTokenApiUrl', $token_api_url);
    }

    /**
     * Method to enqueue JS scripts and CSS of Admin for loading 
     */
    public function enqueue_admin_scripts()
    {
        // Enqueue JS: vf-settings.js
        wp_enqueue_script(
            'vf-settings',
            $this->get_url() . 'js/settings/vf-settings'.self::$vf_file_type.'.js',
            array(),
            filemtime(self::$plugin_directory_path.'js/settings/vf-settings'.self::$vf_file_type.'.js'),
            true
        );

        // Enqueue CSS: vf-settings.css
        wp_enqueue_style(
            'vf_settings_css',
            $this->get_url() . '/css/settings/vf-settings'.self::$vf_file_type.'.css',
            array(),
            filemtime(self::$plugin_directory_path.'css/settings/vf-settings'.self::$vf_file_type.'.css'),
            'screen'
        );
    }

    /**
     * Method to enqueue JS scripts and CSS for loading at Front end
     */
    public function enqueue_frontend_scripts()
    {
        // Enqueue JS: vf.text-library.js
        wp_enqueue_script(
            'vf.text-library',
            $this->get_url() . 'js/vf.text-library'.self::$vf_file_type.'.js',
            array(),
            filemtime(self::$plugin_directory_path.'js/vf.text-library'.self::$vf_file_type.'.js'),
            true
        );

        // Make selected language available to 'vf.text-library.js'
        wp_localize_script( 'vf.text-library', 'vfSelectedLanguage', self::$vf_selected_language);

        // Enqueue JS: vf.speech-handler.js
        wp_enqueue_script(
            'vf.speech-handler',
            $this->get_url() . 'js/vf.speech-handler'.self::$vf_file_type.'.js',
            array(),
            filemtime(self::$plugin_directory_path.'js/vf.speech-handler'.self::$vf_file_type.'.js'),
            true
        );

        // Make images path available to 'vf.speech-handler.js'
        wp_localize_script( 'vf.speech-handler', 'vfImagesPath', self::$vf_url . 'images/');

        // Make ajax obj available to 'vf.speech-handler.js'
        $count_nonce = wp_create_nonce( 'service_call_count' );

        wp_localize_script( 'vf.speech-handler', 'vfAjaxObj', array(
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'nonce'    => $count_nonce,
        ));

        // Make host name available to 'vf.speech-handler.js'
        $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
        $domainName = $_SERVER['SERVER_NAME'];
        wp_localize_script( 'vf.speech-handler', 'vfCurrentHostName', $protocol.$domainName);

        // Enqueue JS: vf.audio-input-handler.js
        wp_enqueue_script(
            'vf.audio-input-handler',
            $this->get_url() . 'js/vf.audio-input-handler'.self::$vf_file_type.'.js',
            array(),
            filemtime(self::$plugin_directory_path.'js/vf.audio-input-handler'.self::$vf_file_type.'.js'),
            true
        );

        // Making API meta data broadcasted in JS file(s)
        self::broadcast_api_meta_data();

        // Enqueue JS: vf.audio-recorder.js
        wp_enqueue_script(
            'vf.audio-recorder',
            $this->get_url() . 'js/recorderjs/vf.audio-recorder'.self::$vf_file_type.'.js',
            array(),
            filemtime(self::$plugin_directory_path.'js/recorderjs/vf.audio-recorder'.self::$vf_file_type.'.js'),
            true
        );

        // Make worker path available to 'vf.audio-recorder.js'
        wp_localize_script( 'vf.audio-recorder', 'vfWorkerPath', $this->get_url(). 'js/recorderjs/vf.audio-recorder-worker'.self::$vf_file_type.'.js');

        // Enqueue JS: voice-forms.js
        wp_enqueue_script(
            'voice-forms',
            $this->get_url() . 'js/voice-forms'.self::$vf_file_type.'.js',
            array(),
            filemtime(self::$plugin_directory_path.'js/voice-forms'.self::$vf_file_type.'.js'),
            true
        );

        // Make floating mic status available to 'voice-forms.js'
        $vf_floating_mic = get_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['floating_mic'], null);
        $vf_floating_mic = self::vf_sanitize_variable_for_local_script($vf_floating_mic);
        wp_localize_script( 'voice-forms', 'vfFloatingMic', $vf_floating_mic);

        // Make button message and talk message available to 'voice-forms.js'
        wp_localize_script( 'voice-forms', 'voice_forms', array(
            'button_message' => __( 'Speech Input', 'voice-forms' ),
            'talk_message'   => __( 'Start Talking…', 'voice-forms' ),
        ));

        // Enqueue CSS: voice-forms.css
        wp_enqueue_style(
            'voice-forms',
            $this->get_url() . 'css/voice-forms'.self::$vf_file_type.'.css',
            array(),
            filemtime(self::$plugin_directory_path.'css/voice-forms'.self::$vf_file_type.'.css'),
            'screen'
        );

        // Get auto timeout value from Database
        $vf_mic_listening_timeout = get_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout'], null);
        $vf_mic_listening_timeout = self::vf_sanitize_variable_for_local_script($vf_mic_listening_timeout);

        if (is_null($vf_mic_listening_timeout)) {
            update_option(Voice_Forms_Settings_Page::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout'], '8');
            $vf_mic_listening_timeout = '8';
        }

        // Make mic auto timeout value and images path available to 'voice-forms.js'
        wp_localize_script( 'voice-forms', 'vfMicListenTimeoutDuration', $vf_mic_listening_timeout);
        wp_localize_script( 'voice-forms', 'vfImagesPath', self::$vf_url . 'images/');

        // Make 'X API Key' available to 'vf.speech-handler.js', 'mvf.audio-input-handler.js' and 'voice-forms.js'
        wp_localize_script( 'vf.speech-handler', 'vfXApiKey', self::$vf_api_access_key);
        wp_localize_script( 'vf.audio-input-handler', 'vfXApiKey', self::$vf_api_access_key);
        wp_localize_script( 'voice-forms', 'vfXApiKey', self::$vf_api_access_key);

        $vf_current_value = get_option('vf_current_value', "0");
        $vf_last_value = get_option('vf_last_value', "0");
        $vf_last_value_updated_at = get_option('vf_last_value_updated_at', null);
        $vf_last_value_updated_at = self::vf_sanitize_variable_for_local_script($vf_last_value_updated_at);

        // Make 'Service logs' available to 'vf.speech-handler.js'
        $vf_service_logs = array(
            'updatedAt' => $vf_last_value_updated_at,
            'currentValue' => $vf_current_value,
            'lastValue' => $vf_last_value,
        );

        wp_localize_script( 'vf.speech-handler', 'vfServiceLogs', $vf_service_logs);
    }

    /**
     * Method to add additional link to settings page below plugin on the plugins page.
     */
    function custom_plugin_row_meta( $links, $file )
    {
        if ( strpos( $file, 'voice-forms.php' ) !== false ) {
            $new_links = array('settings' => '<a href="' . site_url() . '/wp-admin/admin.php?page=voice-forms-settings" title="Voice Forms">Settings</a>');
            $links = array_merge( $links, $new_links );
        }

        return $links;
    }

    /**
     * Class method to get REST API access key ('x-api-key') against license key instate to avail plugin (Voice Forms) service
     *
     * @param $convertable_license_key - String : License key customer posses
     */
    public static function vf_get_api_key_from_license_key($convertable_license_key = null, $license_key_field_changed = false)
    {
        $result = array();

        try {
            // Throw exception when license key is blank or unavailable
            if (!(isset($convertable_license_key) && is_null($convertable_license_key) == false && trim($convertable_license_key) != '')) {
                update_option( 'vf_api_system_key', '');
                throw new Exception("Error: License key is unavailable or invalid.");
            }

            $vf_api_system_key = get_option('vf_api_system_key', null);
            $vf_api_system_key = isset($vf_api_system_key) ? trim($vf_api_system_key) : null;

            if (!empty($vf_api_system_key) && $license_key_field_changed === false) {
                self::$vf_api_access_key = $vf_api_system_key;
            } else {
                $body = array( 'license' => trim($convertable_license_key) );
                $args = array(
                    'body'        => json_encode($body),
                    'timeout'     => '60',
                    'headers'     => array(
                        'Content-Type' => 'application/json',
                        'Accept'       => 'application/json',
                        'x-api-key'    => 'jEODHPKy2z7GEIuerFBWk7a0LqVRJ7ER3aDExmbK'
                    )
                );

                $response = wp_remote_post( 'https://1kosjp937k.execute-api.us-east-1.amazonaws.com/V1', $args );


                // Check the response code
                $response_code = wp_remote_retrieve_response_code( $response );

                if ($response_code == 200) {
                    $response_body = wp_remote_retrieve_body($response);
                    $result = @json_decode($response_body, true);

                    if (!empty($result) && is_array($result)) {
                        if (array_key_exists('errorMessage', $result)) {
                            update_option( 'vf_api_system_key', '');
                        } else {
                            $conversion_status_code = !empty($result['statusCode']) ? trim($result['statusCode']) : null;;
                            $conversion_status      = !empty($result['status']) ? trim($result['status']) : null;

                            if (!is_null($conversion_status_code) && !is_null($conversion_status) && (int)$conversion_status_code == 200 && strtolower(trim($conversion_status)) == 'success') {
                                self::$vf_api_access_key = !empty($result['key']) ? trim($result['key']) : null;
                                
                                if (self::$vf_api_access_key !== null) {
                                    update_option( 'vf_api_system_key', self::$vf_api_access_key);
                                } else {
                                    update_option( 'vf_api_system_key', '');
                                }
                            } else {
                                update_option( 'vf_api_system_key', '');
                            }
                        }
                    }
                }
            }
        } catch (\Exception $ex) {
            self::$vf_api_access_key = null;
        }

        self::$vf_api_access_key = self::vf_sanitize_variable_for_local_script(self::$vf_api_access_key);
    }

    /**
     * Class method to sanitize empty variables
     *
     * @param $vf_var - String : Variable to sanitize
     * @return 
     */
    public static function vf_sanitize_variable_for_local_script($vf_var = null)
    {
        if (empty($vf_var)) {
            return null;
        } else {
            return $vf_var;
        }
    }

    /**
     * Method to log STT service call count to local DB and Cloud
     *
     * @return JSON response obj
     */
    public function vf_log_service_call()
    {
        check_ajax_referer('service_call_count');

        // Get values from database, HTTP request
        $vf_do_update_last_value = isset($_REQUEST['updateLastValue']) ? (int) $_REQUEST['updateLastValue'] : 0;
        $vf_current_value        = (int) get_option('vf_current_value', 0);
        $vf_last_value           = (int) get_option('vf_last_value', 0);
        $vf_last_value_updated_at= get_option('vf_last_value_updated_at', null);
        $vf_current_value_to_log = ($vf_do_update_last_value == 1) ? $vf_current_value : $vf_current_value + 1;
        $vf_temp_last_value      = get_option('vf_last_value', null); // To check if we are making initial service log call
        $vf_log_result = array(
            'vfSttAccess'  => 'allowed',
            'updatedAt'    => $vf_last_value_updated_at,
            'currentValue' => $vf_current_value,
            'lastValue'    => $vf_last_value
        );

        try {
            // We need to reset current value count to 0 if current count log exceeds 25000
            if ($vf_current_value_to_log > 25000) {
                update_option('vf_current_value', 0);
            }

            // Log service count by calling cloud API if last update was before 24 hours or current count is +50 of last count
            if (is_null($vf_temp_last_value) || $vf_do_update_last_value === 1 || ($vf_current_value_to_log > ($vf_last_value + 50))) {
                $vf_body = array(
                    'license'      => trim(self::$vf_license_key),
                    'action'       => "logCalls",
                    'currentValue' => $vf_current_value_to_log,
                    'lastValue'    => $vf_last_value,
                );
                
                $vf_args = array(
                    'body'         => json_encode($vf_body),
                    'timeout'      => '60',
                    'headers'      => array(
                        'Content-Type' => 'application/json',
                        'Accept'       => 'application/json',
                        'x-api-key'    => 'jEODHPKy2z7GEIuerFBWk7a0LqVRJ7ER3aDExmbK'
                    )
                );

                $vf_response = wp_remote_post( 'https://1kosjp937k.execute-api.us-east-1.amazonaws.com/V2', $vf_args );

                // Check the response code
                $vf_response_code = wp_remote_retrieve_response_code($vf_response);


                if ($vf_response_code == 200) {
                    $vf_response_body = wp_remote_retrieve_body($vf_response);
                    $vf_result = @json_decode($vf_response_body, true);

                    if (!empty($vf_result) && is_array($vf_result)) {
                        $log_status = array_key_exists("status",$vf_result) ? strtolower($vf_result['status']) : 'failed';
                        $actual_current_value = array_key_exists("current Value",$vf_result) ? strtolower($vf_result['current Value']) : null;
                        $vf_error = array_key_exists("errorMessage",$vf_result) ? true : false;

                        if ($log_status == 'success' && is_null($actual_current_value) === false && $vf_error === false) {
                            // Store updated values to database
                            $vf_current_timestamp = time(); // epoc 
                            update_option('vf_current_value', $actual_current_value);
                            update_option('vf_last_value', $actual_current_value);
                            update_option('vf_last_value_updated_at', $vf_current_timestamp);
                            
                            // Prepare response 
                            $vf_log_result['updatedAt']    = $vf_current_timestamp;
                            $vf_log_result['currentValue'] = $actual_current_value;
                            $vf_log_result['lastValue']    = $actual_current_value;
                            $vf_log_result['cloud']    = true;
                        }
                    }
                } 
            } else {
                // Icrease current count
                update_option('vf_current_value', $vf_current_value_to_log);

                // Prepare response
                $vf_log_result['currentValue'] = $vf_current_value_to_log;
                $vf_log_result['local']    = true;
            }
        } catch (\Exception $ex) {
            // Prepare response 
            $vf_log_result['vfSttAccess']  = 'restricted';
        }

        wp_send_json($vf_log_result);
    }

    /**
     * Method to register plugin for the first time
     *
     */
    public static function vf_register_plugin()
    {
        try {           
            // Get plugin first activation status and license key from DB 
            $vf_license_key      = get_option('vf_license_key', null);
            $vf_first_activation = get_option('vf_first_activation', null);   
            $vf_site_name        = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];
            
            if (empty($vf_first_activation) && empty(trim($vf_license_key))) {
                // Mark first activation activity flag in local DB 
                update_option('vf_first_activation', true);// Store first activation flag in DB

                // Generate UUID and store in DB
                $vf_new_uuid = wp_generate_uuid4();
                update_option('vf_uuid', $vf_new_uuid);

                $vf_body = array(
                    'action' => 'regVF',                  
                    'url'    => $vf_site_name.'_'.$vf_new_uuid,
                );

                $vf_args = array(
                    'body'        => json_encode($vf_body),
                    'timeout'     => '60',
                    'headers'     => array(
                        'Content-Type' => 'application/json',
                        'Accept'       => 'application/json',
                        'x-api-key'    => 'jEODHPKy2z7GEIuerFBWk7a0LqVRJ7ER3aDExmbK'
                    )
                );

                $vf_response = wp_remote_post( 'https://1kosjp937k.execute-api.us-east-1.amazonaws.com/V2', $vf_args );

                // Check the response body
                $vf_response_body = wp_remote_retrieve_body($vf_response);
                $vf_result = @json_decode($vf_response_body, true);             

                if (!empty($vf_result) && is_array($vf_result)) {
                    $log_status = array_key_exists('status', $vf_result) ? strtolower(trim($vf_result['status'])) : null;

                    if ($log_status == '200 success') {
                        // Do nothing for now                               
                    }  
                }             
            }
        } catch(\Exception $ex) {
            // Do nothing for now               
        }
    }
}
