// *****************************************************************************************************
// *******              speak2web VOICE FORMS                                                ***********
// *******               Get your subscription at                                            ***********
// *******                    https://speak2web.com/plugin#plans                             ***********
// *******               Need support? https://speak2web.com/support                         ***********
// *******               Licensed GPLv2+                                                     ***********
//******************************************************************************************************

(function () {
    'use strict';
    
    // Auto timeout duration to stop listening Mic
    var vfMicListenAutoTimeoutDuration = null;
    let vfOtherInputTimeoutDuration = null;
    let vfTimerID = 0;
    let vfTextareaTimeoutDuration = 20 * 1000;

    if (typeof(vfMicListenTimeoutDuration) != 'undefined' && vfMicListenTimeoutDuration !== null) {
        vfOtherInputTimeoutDuration = parseInt(vfMicListenTimeoutDuration);
        vfOtherInputTimeoutDuration = isNaN(vfOtherInputTimeoutDuration) ? 8 : vfOtherInputTimeoutDuration;
    } else {
        vfOtherInputTimeoutDuration = 8;
    }

    vfOtherInputTimeoutDuration = (vfOtherInputTimeoutDuration < 8) ? 8 : vfOtherInputTimeoutDuration;
    vfOtherInputTimeoutDuration = (vfOtherInputTimeoutDuration > 20) ? 20 : vfOtherInputTimeoutDuration;
    vfOtherInputTimeoutDuration = vfOtherInputTimeoutDuration * 1000;

    // Local function to clear interval
    function vfClearTimer(timerInstance) {
        try {
            if (typeof (timerInstance) != 'undefined' && timerInstance !== null && timerInstance != false) {
                clearInterval(timerInstance);
            }
        } catch (err) {/* do nothing for now*/ }
    }

    // sanitize alpha-numeric css values to get numeric value
    function getNumber(number) {
        number = parseInt(number, 10);
        return isNaN(number) || number === null || typeof (number) === 'undefined' ? 0 : number;
    }

    // Function to check if any mic already listening
    function vfAnyOtherMicListening(vfExceptionBtnId = null) {
        var vfOneOfMicListening = false;
        try {
            var vfAllMicButtons = document.querySelectorAll('button.voice-forms-button');

            if (typeof(vfAllMicButtons) == 'undefined' 
                || vfAllMicButtons === null 
                || vfExceptionBtnId == null) { return vfOneOfMicListening; }

            for (var vfI = 0; vfI < vfAllMicButtons.length; vfI++) {
                var vfClassNames = vfAllMicButtons[vfI].className;
                var vfBtnId = vfAllMicButtons[vfI].getAttribute('id');

                if (!(typeof(vfClassNames) != 'undefined' && vfClassNames.trim() != '')) continue;

                if (vfClassNames.indexOf('listening') != -1 && vfExceptionBtnId != vfBtnId) {
                    vfOneOfMicListening = true;
                    break;
                }
            }
        } catch (err) {
            vfOneOfMicListening = false;
        }

        return vfOneOfMicListening;
    }

    var typeOfInputFieldsToSeek = ['text', 'email', 'search'];

    // Detect and decide event handler for browsers over iOS
    var vfIOS = !!navigator.platform && /iPad|iPhone|iPod/.test(navigator.platform);
    var vfMicEventToListen = (vfIOS === true) ? 'touchend' : 'click';

    // Detect Android OS
    var ua = navigator.userAgent.toLowerCase();
    var isAndroid = ua.indexOf("android") > -1;

    var recordTimer = null;

    var speechInputWrappers = document.querySelectorAll('form');// Get all forms on a page
    let formElementForWidget = null;

    [].forEach.call(speechInputWrappers, function (speechInputWrapper, index) {
        try {
            // Try to show the form temporarily so we can calculate the sizes
            var speechInputWrapperStyle = speechInputWrapper.getAttribute('style');
            var havingInlineStyle = (typeof(speechInputWrapperStyle) != 'undefined' 
            && speechInputWrapperStyle !== null && speechInputWrapperStyle.trim() !== '') ? true : false;
            speechInputWrapperStyle = (havingInlineStyle) ? speechInputWrapperStyle + ';' : '';
            speechInputWrapper.setAttribute('style', speechInputWrapperStyle + 'display: block !important');
            speechInputWrapper.classList.add('voice-forms-wrapper');
            speechInputWrapper.classList.add('vf-sanitize-form-wrapper');
            
            let isSearchForm = false;
            var recognizing = false;
            let roleOfForm = speechInputWrapper.getAttribute('role') || "";
            let classesOfForm = speechInputWrapper.classList;
            let allInputElements = speechInputWrapper.querySelectorAll('input:not([type=hidden]), button[type=submit], textarea');

            if (roleOfForm.toLowerCase() === 'search' || classesOfForm.contains('searchform') 
                || classesOfForm.contains('search_form') || classesOfForm.contains('search-form') 
                || classesOfForm.contains('searchForm')) {
                isSearchForm =  true;
            }

            // Preserve first form on page and it's input element for widget
            if (index == 0) {
                formElementForWidget = speechInputWrapper;
            }

            [].forEach.call(allInputElements, function (inputElement, inputIndex) {
                var inputEl = null;
                let inputType = inputElement.getAttribute('type') || "";
                let classesOfInput = inputElement.className || "";
                let idOfInputElement = inputElement.getAttribute('id') || null;
                let tagNameOfInputElement = inputElement.tagName;

                if (idOfInputElement && idOfInputElement !== null) {
                    idOfInputElement = idOfInputElement.toLowerCase();
                }

                if (classesOfInput && classesOfInput !== null) {
                    classesOfInput = classesOfInput.toLowerCase();
                }

                // Check if input marked with keywords related to date
                let isDateField = false;

                // Check for search form
                if (classesOfInput.indexOf('datepicker') !== -1 
                    || classesOfInput.indexOf('date') !== -1 
                    || idOfInputElement === 'datepicker' 
                    || idOfInputElement === 'date') {
                    isDateField = true;
                }

                // check if input field is intented for date picking
                if (classesOfInput.indexOf('datepicker') === -1 
                    && idOfInputElement !== 'datepicker' 
                    && classesOfInput.search(/validat|candidat/ig) !== -1) {
                    isDateField = false;
                }

                // check if text field is intended for email
                if (inputType === 'text' && classesOfInput.search(/email/ig) !== -1) {
                    inputType = 'email';
                }

                 // Check textarea/large input box
                let isTextArea = (tagNameOfInputElement.toLowerCase() === 'textarea') ? true : false;
                vfMicListenAutoTimeoutDuration =  isTextArea == true ? vfTextareaTimeoutDuration : vfOtherInputTimeoutDuration;

                if ((typeOfInputFieldsToSeek.includes(inputType) && isDateField == false) || isTextArea == true ){
                    inputEl = inputElement;
                } else if (inputType.toLowerCase() == 'submit') {
                    // Restrict form submission if mic is recording
                    speechInputWrapper.addEventListener('submit', function(event) {
                        if (recognizing == true) {
                            event.preventDefault();
                        }
                    }, false);

                    // Remove any overlapping icon from submit button of search form
                    if (isSearchForm === true) {
                        try {
                            let submitButtonChildNodes = inputElement.querySelectorAll('img, svg');
                            
                            for (let j = 0; j < submitButtonChildNodes.length; j++) {
                                let submitBtnChildNode = submitButtonChildNodes[j];
                                submitBtnChildNode.classList.add('vf-hide-element');
                            }
                        } catch(err) {
                            // do nothing for now
                        }
                    }
                }
            
                // If search input field not found then continue
                if (null === inputEl) { return true; }

                // Add some markup as a button to the search form
                var micBtn = document.createElement('button');
                micBtn.setAttribute('type', 'button');
                micBtn.setAttribute('class', 'voice-forms-button');
                micBtn.setAttribute('id', 'voice-forms-button' + inputIndex);
                micBtn.appendChild(document.createTextNode(voice_forms.button_message));

                // Add mic image icon
                var vfMicIcon = document.createElement('img');
                vfMicIcon.setAttribute('src', vfImagesPath + 'vf_mic.svg');
                vfMicIcon.setAttribute('class', 'vf-mic-image');

                micBtn.appendChild(vfMicIcon);

                var inputHeight = getNumber(inputEl.offsetHeight);// Get search input height
                var buttonSize = getNumber(0.8 * inputHeight);

                // Set default mic button size to 35px when button size calculated to 0 or unknown
                if (getNumber(buttonSize) == 0) { inputHeight = buttonSize = 35; }

                var micbtnPositionTop = getNumber(0.1 * inputHeight);

                // For textarea/large input
                if (isTextArea === true) {
                    micbtnPositionTop = 10;
                    buttonSize = 35;
                }

                // Size and position of complete mic button
                var inlineStyle = 'top: ' + micbtnPositionTop + 'px; ';
                inlineStyle += 'height: ' + buttonSize + 'px !important; ';
                inlineStyle += 'width: ' + buttonSize + 'px !important; ';
                inlineStyle += 'z-index: 999 !important; margin-left: 3px !important; border-radius: 50% !important; border: 2px solid #ffff !important;';
                micBtn.setAttribute('style', inlineStyle);

                // Create Wrapper to wrap around input search field like a elastic band
                var wrapper = document.createElement('div');
                let inputCurrentStyle = window.getComputedStyle(inputEl);
                wrapper.setAttribute('class', 'vf-mic-band');
                wrapper.setAttribute('onclick', 'return false');
                wrapper.style.width = inputCurrentStyle.width;
                inputEl.insertAdjacentElement('beforebegin', wrapper);// Place wrapper before input search field

                // Set parent element's (parent of inputEl) display stack order higher 
                // To handle overlapped submit button on mic icon
                var parentEl = inputEl.parentNode.nodeName;

                if (typeof(parentEl) != 'undefined' && parentEl !== null && parentEl.length != 0) {
                    parentEl = parentEl.toLowerCase();

                    if (parentEl != 'form') {
                        inputEl.parentNode.style.zIndex = 1;
                    }
                }

                // Append search input field element inside a wrapper band
                wrapper.appendChild(inputEl);

                // Place mic button/icon exact before search input field element
                inputEl.insertAdjacentElement('beforebegin', micBtn);
                inputEl.classList.add('vf-mic-band');
                
                // Setup recognition
                var finalTranscript = '';                
                var final_transcript = "";
                var ignore_onend;

                if ('webkitSpeechRecognition' in window) {
                    var recognition = new webkitSpeechRecognition();
                    recognition.continuous = true;
                    recognition.interimResults = true;

                    recognition.onstart = function () {
                        recognizing = true;
                    };

                    recognition.onerror = function (event) {
                      //  vfClearTimer(recordTimer);
                       // recordTimer = false;
                        micBtn.classList.remove('listening');
                        recognizing = false;

                        if (event.error == 'no-speech') {
                            inputEl.placeholder = vfAlternativeResponse['notAudible'];
                            tts(vfAlternativeResponse['notAudible'], function(){});
                            ignore_onend = true;
                        }
                        if (event.error == 'audio-capture') {
                            inputEl.placeholder = vfMessages['micNotAccessible'];
                            ignore_onend = true;
                        }
                        if (event.error == 'not-allowed') {
                            inputEl.placeholder = vfMessages['browserDenyMicAccess'];
                            micBtn.style.setProperty("color", "white");
                            ignore_onend = true;
                        }
                    };

                    function processEnd() {
                        recognizing = false;

                        if (ignore_onend) { return; }
                        
                        micBtn.classList.remove('listening');
                        micBtn.style.setProperty("color", "white");               
                        
                        if (typeof(finalTranscript) != 'undefined' && finalTranscript.length != 0) {
                            let transcribedText = finalTranscript;

                            // Sanitize email before putting transcribed text in input field
                            if (inputType === 'email') {
                                transcribedText = vfFormatEmail(transcribedText);
                            }


                            let newChromeTranscribedText = transcribedText && transcribedText !== null ? transcribedText : finalTranscript;

                            if (isTextArea === true) {
                                // Preserve previous input value for textarea and append new value
                                inputEl.value += ' ' + newChromeTranscribedText;
                            } else {
                                inputEl.value = newChromeTranscribedText;
                            }

                            if (isSearchForm == true) {
                                tts(vfAlternativeResponse['basic'], function () {
                                    speechInputWrapper.submit();
                                });
                            }
                        } else {
                            // tts(vfAlternativeResponse['notAudible'], function(){});
                            inputEl.placeholder = vfMessages['ask'];
                        }
                    };

                    recognition.onend = function () {
                        if (isAndroid) {
                            processEnd();
                        }
                    };

                    recognition.onresult = function (event) {
                        let interim_transcript = '';
                        
                        if (typeof (event.results) == 'undefined') {
                            recognition.onend = null;
                            recognition.stop();
                            inputEl.placeholder = vfMessages['unableToHear'];
                            tts(vfAlternativeResponse['micConnect'], function () { });
                            return;
                        }

                        for (var i = event.resultIndex; i < event.results.length; ++i) {
                            if (event.results[i].isFinal) {
                                finalTranscript = event.results[i][0].transcript;

                                if (isAndroid == false) {
                                    processEnd();
                                    recognition.stop();
                                }                        
                            } else {
                                if (isTextArea === false) {
                                    interim_transcript += event.results[i][0].transcript;
                                    inputEl.value = interim_transcript;
                                }                       
                            }
                        }
                    };

                    micBtn.addEventListener(vfMicEventToListen, function (event) {
                        // micBtn.onclick = function (event) {
                        if (vfAnyOtherMicListening(micBtn.getAttribute('id')) === true) return;

                        // Deny recording if microphone is not accessible
                        var classNamesInContext = micBtn.className;

                        //We need to make sure that we keep track which timer is firing!
                        //To keep do so we start the timer in this function and we keep an ID in local storage to ensure we know if we are still active
                        function vfSetTimeOut(vfMyTimerid, vFMicBtn, vfTimeOutTime) {
                            return new Promise(function (resolve) {
                                setTimeout(function () {
                                    var updatedClassNames = vFMicBtn.className;

                                    // Trigger a click to reset and end recording. As user did not end manually.
                                    if (typeof (updatedClassNames) != 'undefined' && updatedClassNames !== null
                                        && updatedClassNames.indexOf('listening') != -1 && vfMyTimerid == vfTimerID) {
                                        vFMicBtn.click();
                                    }
                                    resolve('resolved');
                                }, vfTimeOutTime);
                            });
                        }
                                
                        // Set interval for 20 seconds when mic is in reset/fresh state ie: when user clicked mic for recording to start
                        if (!(typeof (classNamesInContext) != 'undefined' && classNamesInContext !== null
                            && classNamesInContext.indexOf('listening') != -1)) {
                            //recordTimer = setInterval(function () {      
                            vfTimerID = vfTimerID + 1;
                            vfSetTimeOut(vfTimerID,micBtn, inputEl.tagName.toLowerCase() == 'textarea' ? vfTextareaTimeoutDuration : vfOtherInputTimeoutDuration);
                        }

                        if (recognizing) {
                            if (isAndroid == false) {
                                processEnd();
                                recognition.stop();
                            }
                        } else {
                            micBtn.classList.add('listening');
                            event.preventDefault();
                            tts_stop();
                            finalTranscript = '';

                            if (isTextArea === false) {
                                inputEl.value = '';
                            }
                            recognizing = true;                            
                            recognition.lang = vfSelectedLang;
                            recognition.start();
                            ignore_onend = false;
                        }
                    });
                } else {
                    //CODE FOR BROWSERS THAT DO NOT SUPPORT STT NATIVLY
                    // MUST USE THE BUILT IN MICROPHONE
                    micBtn.addEventListener(vfMicEventToListen, function (event) {
                        if (vfAnyOtherMicListening(micBtn.getAttribute('id')) === true) return;

                        // Deny recording if microphone is not accessible
                        if (!vfAudioRecorder || !vfAudioContext) {
                            vfInitAudio(function (a) {
                                if (!vfAudioRecorder || !vfAudioContext) {
                                    alert(vfMessages['cantAccessMicrophone']);
                                    return false;
                                } else {
                                    listenEvent();
                                }
                            });
                        } else {
                            listenEvent();
                        }

                        function listenEvent() {
                            // If API system key is unavailable then acknowledge service unavailability and stop voice navigation.
                            if (!(typeof (vfXApiKey) != 'undefined' && vfXApiKey !== null)) {
                                tts(vfAlternativeResponse['unavailable'], function () { });
                                return false;
                            }

                            //We need to make sure that we keep track which timer is firing!
                            //To keep do so we start the timer in this function and we keep an ID in local storage to ensure we know if we are still active
                            function vfSetTimeOut(vfMyTimerid, vFMicBtn, vfTimeOutTime) {
                                return new Promise(function (resolve) {
                                    setTimeout(function () {
                                        var updatedClassNames = vFMicBtn.className;

                                        // Trigger a click to reset and end recording. As user did not end manually.
                                        if (typeof (updatedClassNames) != 'undefined' && updatedClassNames !== null
                                            && updatedClassNames.indexOf('listening') != -1 && vfMyTimerid == vfTimerID) {
                                            vFMicBtn.click();
                                        }
                                        resolve('resolved');
                                    }, vfTimeOutTime);
                                });
                            }

                            // Get current list of class attached with interacted mic button
                            var classNamesInContext = micBtn.className;

                            // Set interval for 20 seconds when mic is in reset/fresh state ie: when user clicked mic for recording to start
                            if (!(typeof (classNamesInContext) != 'undefined' && classNamesInContext !== null
                                && classNamesInContext.indexOf('listening') != -1)) {
                                //recordTimer = setInterval(function () {
                                vfTimerID = vfTimerID + 1;
                                vfSetTimeOut(vfTimerID,micBtn, inputEl.tagName.toLowerCase() == 'textarea' ? vfTextareaTimeoutDuration : vfOtherInputTimeoutDuration);
                            }

                            // User ending recording by clicking back mic
                            if (recognizing) {
                                // clear the record timer interval when user ending recording
                                //vfClearTimer(recordTimer);
                                //recordTimer = false;

                                // Stop recorder
                                vfAudioRecorder.stop();

                                // Stop access to audio resource
                                vfStopAudio();

                                //replace recording with mic icon
                                micBtn.classList.remove('listening');

                                micBtn.style.setProperty("color", "white");
                                inputEl.placeholder = vfMessages['transcribeText'];

                                tts("", function () {
                                    vfAudioRecorder.getBuffers(function (buffers) {
                                        vfAudioRecorder.exportWAV(function (blob) {
                                            vfRecIndex++;

                                            stt(blob, false, function (request) {
                                                if (typeof(request) != 'undefined' && request.length != 0) {
                                                    let transcribedText = request;

                                                    // Sanitize email before putting transcribed text in input field
                                                    if (inputType === 'email') {
                                                        transcribedText = vfFormatEmail(transcribedText);
                                                    }
                                                    let newNonChromeTranscribedText = transcribedText && transcribedText !== null ? transcribedText : request;

                                                    if (isTextArea === true) {
                                                        // Preserve previous input value for textarea and append new value
                                                        inputEl.value += ' ' + newNonChromeTranscribedText;
                                                    } else {
                                                        inputEl.value = newNonChromeTranscribedText;
                                                    }
                                                    
                                                    if (isSearchForm == true) {
                                                        tts(vfAlternativeResponse['basic'], function(){
                                                            speechInputWrapper.submit();
                                                        });
                                                    }
                                                } else {
                                                    tts(vfAlternativeResponse['notAudible'], function(){});
                                                    inputEl.placeholder = vfMessages['ask'];
                                                }
                                            });
                                        });
                                    });
                                });

                                recognizing = false;
                                return;
                            } else {// User started recording by clicking mic
                                micBtn.classList.add('listening');
                                event.preventDefault();

                                tts_stop();
                                inputEl.value = finalTranscript = '';

                                recognizing = true;
                                vfAudioRecorder.clear();
                                vfAudioRecorder.record(micBtn, inputEl.tagName);
                            }
                        }
                    }, false); 
                }
            });

            // Reset form style again
            speechInputWrapper.setAttribute('style', speechInputWrapperStyle);
        } catch (err) {  /* do nothing */ }
    });
    
    // Load dloating mic with search bar
    if (vfFloatingMic && vfFloatingMic === 'yes') {
        //############################# Floating mic - Widget ###################################
        let vfDocFragment = document.createDocumentFragment();
        // Create root/widget wrapper
        let vfWidgetWrapper = document.createElement('div');
        vfWidgetWrapper.setAttribute('class', 'vf-widget-wrapper');

        // Create chat wrapper
        let vfWidgetChatWrapper = document.createElement('div');
        vfWidgetChatWrapper.setAttribute('class', 'vf-widget-chat-wrapper');

        // ############################ Widget Fields (Input section) ############################
        // Create widget text field and mic (Input Section)
        let vfWidgetField = document.createElement('div');
        vfWidgetField.setAttribute('class', 'vf-widget-field');

        // Create mic icon wrapper
        let vfWidgetMic = document.createElement('a');
        vfWidgetMic.setAttribute('id', 'vfWidgetMic');
        vfWidgetMic.setAttribute('class', 'vf-widget-button');

        // Create and append mic icon/image to mic wrapper
        let vfWidgetMicImg = document.createElement('img');
        vfWidgetMicImg.setAttribute('src', vfImagesPath + 'vf-widget-mic-black.svg');
        vfWidgetMic.appendChild(vfWidgetMicImg);

        // Create button wrapper next to input text field
        let vfWidgetSearchBtn = document.createElement('a');
        vfWidgetSearchBtn.setAttribute('id', 'vfWidgetSearchBtn');

        // Create and append search button to button wrapper
        let vfWidgetSearchBtnEl = document.createElement('button');
        vfWidgetSearchBtnEl.setAttribute('class', 'vf-widget-form-submit-btn');
        vfWidgetSearchBtnEl.setAttribute('type', 'submit');
        vfWidgetSearchBtnEl.setAttribute('alt', 'Go');
        vfWidgetSearchBtnEl.setAttribute('title', 'Search');
        vfWidgetSearchBtn.appendChild(vfWidgetSearchBtnEl);

        // Create form for widget
        let vfWidgetForm = document.createElement('form');
        vfWidgetForm.setAttribute("class", "vf-widget-form");

        if (formElementForWidget !== null) {
            vfWidgetForm.action = formElementForWidget.action;
            vfWidgetForm.method = formElementForWidget.method;
        } else {
            vfWidgetForm.action = vfGetCurrentHostURL() + '/';
            vfWidgetForm.method = "get";
        }

        // Create input text field 
        let vfWidgetSearch = document.createElement('input');
        vfWidgetSearch.setAttribute('id', 'vfWidgetSearch');
        vfWidgetSearch.setAttribute('class', 'vf-widget-search vf-widget-search-text');
        vfWidgetSearch.setAttribute('name', 'vf-widget-search');
        vfWidgetSearch.setAttribute('placeholder', vfWidgetMessages['placeholder']);
        vfWidgetSearch.setAttribute('name', 's');

        vfWidgetForm.appendChild(vfWidgetSearch);
        vfWidgetForm.appendChild(vfWidgetSearchBtn);

        // Append mic and form to widget field section (input section)
        vfWidgetField.appendChild(vfWidgetMic);
        vfWidgetField.appendChild(vfWidgetForm);

        // Append chat header, chat conversation and input fields to widget chat wrapper
        vfWidgetChatWrapper.appendChild(vfWidgetField);

        // ################################ Widget Toggle button #########################
        // Create a widget toggle button wrapper
        let vfWidgetToggleButton = document.createElement('a');
        vfWidgetToggleButton.setAttribute('id', 'vfWidgetToggleButton');
        vfWidgetToggleButton.setAttribute('class', 'vf-widget-button');

        // Create toggle button icon element
        let vfWidgetIcon = document.createElement('div');
        vfWidgetIcon.setAttribute('class', 'vf-widget-icon vf-widget-toggle-button vf-toggle-btn-mic');

        // Append toggle button icon to toggle button wrapper
        vfWidgetToggleButton.appendChild(vfWidgetIcon);

        // Append chat wrapper and toggle button to widget wrapper
        vfWidgetWrapper.appendChild(vfWidgetChatWrapper);
        vfWidgetWrapper.appendChild(vfWidgetToggleButton);

        // Append widget to body
        vfDocFragment.appendChild(vfWidgetWrapper);
        document.body.appendChild(vfDocFragment);

        // Listen event to show/hide widget
        vfWidgetToggleButton.addEventListener('click', function(event){
            vfToggleWidgetElements();
        });

        /*############################# Widget mic handling ################################*/
        // Setup recognition
        let widgetFinalTranscript = '';
        let widgetRecognizing     = false;
        let widget_final_transcript = "";
        let widget_ignore_onend;

        if ('webkitSpeechRecognition' in window) {
            let widgetRecognition = new webkitSpeechRecognition();
            widgetRecognition.continuous = true;
            widgetRecognition.interimResults = true;

            widgetRecognition.onstart = function () { 
                widgetRecognizing = true; 
            };

            widgetRecognition.onerror = function (event) {
                vfWidgetMic.classList.remove('listening');
                widgetRecognizing = false;

                if (event.error == 'no-speech') {
                    tts(vfAlternativeResponse['notAudible'], function(){});
                    widget_ignore_onend = true;
                    vfWidgetSearch.placeholder = vfAlternativeResponse['notAudible'];
                }

                if (event.error == 'audio-capture') {
                    widget_ignore_onend = true;
                    vfWidgetSearch.placeholder = vfMessages['micNotAccessible'];
                }

                if (event.error == 'not-allowed') {
                    widget_ignore_onend = true;
                    vfWidgetSearch.placeholder = vfMessages['browserDenyMicAccess'];
                }
            };

            function widgetProcessEnd() {
                widgetRecognizing = false;

                if (widget_ignore_onend) { return; }

                widgetFinalTranscript = widget_final_transcript;   
                vfWidgetMic.classList.remove('listening');
                
                if (typeof(widgetFinalTranscript) != 'undefined' && widgetFinalTranscript.length != 0) {
                    vfWidgetSearch.value = widgetFinalTranscript;
                    
                    tts(vfAlternativeResponse['basic'], function () {
                        vfWidgetForm.submit();
                    });
                } else {
                    tts(vfAlternativeResponse['notAudible'], function(){});
                    vfWidgetSearch.placeholder = vfMessages['ask'];
                }

            }

            widgetRecognition.onend = function () { 
                if (isAndroid) { widgetProcessEnd(); }
            };

            widgetRecognition.onresult = function (event) {
                let interim_transcript = '';

                if (typeof (event.results) == 'undefined') {
                    widgetRecognition.onend = null;
                    widgetRecognition.stop();
                    vfWidgetSearch.placeholder = vfMessages['unableToHear'];
                    tts(vfAlternativeResponse['micConnect'], function () { });
                    return;
                }

                let eventResultsLength = event.results.length;

                for (let i = event.resultIndex; i < eventResultsLength; ++i) {
                    if (event.results[i].isFinal) {
                        widget_final_transcript = event.results[i][0].transcript;

                        if (isAndroid == false) {
                            widgetProcessEnd();
                            widgetRecognition.stop();
                        }                        
                    } else {
                        interim_transcript += event.results[i][0].transcript;
                    }
                }
            };

            vfWidgetMic.addEventListener(vfMicEventToListen, function (event) {
                if (vfAnyOtherMicListening(vfWidgetMic.getAttribute('id'), vfWidgetMic) === true) return;

                // Deny recording if microphone is not accessible
                let classNamesInContext = vfWidgetMic.className;

                //We need to make sure that we keep track which timer is firing!
                //To keep do so we start the timer in this function and we keep an ID in local storage to ensure we know if we are still active
                function vfSetTimeOut(vfMyTimerid, vfMicBtn, vfTimeOutTime) {
                    return new Promise(function (resolve) {
                        setTimeout(function () {
                            var updatedClassNames = vfMicBtn.className;

                            // Trigger a click to reset and end recording. As user did not end manually.
                            if (typeof (updatedClassNames) != 'undefined' && updatedClassNames !== null
                                && updatedClassNames.indexOf('listening') != -1 && vfMyTimerid == vfTimerID) {
                                vfMicBtn.click();
                            }
                            resolve('resolved');
                        }, vfTimeOutTime);
                    });
                }

                // Set interval for 20 seconds when mic is in reset/fresh state ie: when user clicked mic for recording to start
                if (!(typeof (classNamesInContext) != 'undefined' && classNamesInContext !== null
                    && classNamesInContext.indexOf('listening') != -1)) {
                    vfTimerID = vfTimerID + 1;
                    vfSetTimeOut(vfTimerID, vfWidgetMic, vfOtherInputTimeoutDuration);
                }

                if (widgetRecognizing) {
                    if (isAndroid == false) {
                        widgetProcessEnd();
                        widgetRecognition.stop();
                    }
                } else {
                    vfWidgetMic.classList.add('listening');
                    event.preventDefault();
                    tts_stop();
                    widgetFinalTranscript = '';
                    widgetRecognizing = true;
                    widgetRecognition.lang = vfSelectedLang;
                    widgetRecognition.start();
                    widget_ignore_onend = false;
                }
            });
        } else {
            //CODE FOR BROWSERS THAT DO NOT SUPPORT STT NATIVLY
            // MUST USE THE BUILT IN MICROPHONE
            vfWidgetMic.addEventListener(vfMicEventToListen, function (event) {
                if (vfAnyOtherMicListening(vfWidgetMic.getAttribute('id'), vfWidgetMic) === true) return;

                // Deny recording if microphone is not accessible
                if (!vfAudioRecorder || !vfAudioContext) {
                    vfInitAudio(function (a) {
                        if (!vfAudioRecorder || !vfAudioContext) {                        
                            vfWidgetSearch.placeholder = vfMessages['micNotAccessible'];
                            return false;
                        } else {
                            widgetListenEvent();
                        }
                    });
                } else {
                    widgetListenEvent();
                }

                function widgetListenEvent() {
                    // If API system key is unavailable then acknowledge service unavailability and stop voice navigation.
                    if (!(typeof (vfXApiKey) != 'undefined' && vfXApiKey !== null)) {
                        tts(vfAlternativeResponse['unavailable'], function () { });
                        return false;
                    }

                    // Get current list of class attached with interacted mic button
                    let classNamesInContext = vfWidgetMic.className;

                    //We need to make sure that we keep track which timer is firing!
                    //To keep do so we start the timer in this function and we keep an ID in local storage to ensure we know if we are still active
                    function vfSetTimeOut(vfMyTimerid, vfMicBtn, vfTimeOutTime) {
                        return new Promise(function (resolve) {
                            setTimeout(function () {
                                var updatedClassNames = vfMicBtn.className;

                                // Trigger a click to reset and end recording. As user did not end manually.
                                if (typeof (updatedClassNames) != 'undefined' && updatedClassNames !== null
                                    && updatedClassNames.indexOf('listening') != -1 && vfMyTimerid == vfTimerID) {
                                    vfWidgetMic.click();
                                }
                                resolve('resolved');
                            }, vfTimeOutTime);
                        });
                    }

                    // Set interval for 20 seconds when mic is in reset/fresh state ie: when user clicked mic for recording to start
                    if (!(typeof (classNamesInContext) != 'undefined' && classNamesInContext !== null 
                        && classNamesInContext.indexOf('listening') != -1)) {
                        vfTimerID = vfTimerID + 1;
                        vfSetTimeOut(vfTimerID, vfWidgetMic, vfOtherInputTimeoutDuration);
                    }

                    // User ending recording by clicking back mic
                    if (widgetRecognizing) {
                        // Stop recorder
                        vfAudioRecorder.stop();

                        // Stop access to audio resource
                        vfStopAudio();

                        //replace recording with mic icon
                        vfWidgetMic.classList.remove('listening');

                        vfWidgetSearch.placeholder = vfMessages['transcribeText'];

                        tts("", function () {
                            vfAudioRecorder.getBuffers(function (buffers) {
                                vfAudioRecorder.exportWAV(function (blob) {
                                    vfRecIndex++;
                                
                                    stt(blob, false, function (request) {
                                        if (typeof(request) != 'undefined' && request.length != 0) {
                                            vfWidgetSearch.value = request;

                                            tts(vfAlternativeResponse['basic'], function(){
                                                vfWidgetForm.submit();
                                            });
                                        } else {
                                            tts(vfAlternativeResponse['notAudible'], function(){});
                                            vfWidgetSearch.placeholder = vfMessages['ask'];
                                        }
                                    });
                                });
                            });
                        });

                        widgetRecognizing = false;
                        return;
                    } else {// User started recording by clicking mic
                        vfWidgetMic.classList.add('listening');
                        event.preventDefault();

                        tts_stop();
                        widgetFinalTranscript = '';

                        widgetRecognizing = true;
                        vfAudioRecorder.clear();
                        vfAudioRecorder.record(vfWidgetMic);
                    }
                }
            }, false); 
        }

        /*###############################################################################*/

        /**
         * Function to toggle class of the HTML element
         *
         * @param {elmSelector - String} : CSS Selector
         * @param {nameOfClass - String} : Class name to add/remove
         */
        function vfToggleClass(elmSelector, nameOfClass) {
            if (!(typeof(elmSelector) != 'undefined' && elmSelector != null && elmSelector.length != 0)) return false;

            let element = document.querySelector(elmSelector);

            if (element.classList) { 
                element.classList.toggle(nameOfClass);
            } else {
                // For IE9

                let classes = element.className.split(" ");
                let i = classes.indexOf(nameOfClass);

                if (i >= 0) {
                    classes.splice(i, 1);
                } else {
                    classes.push(nameOfClass);
                    element.className = classes.join(" "); 
                }
            }
        }

        /**
         * Function to toggle chat and links
         */
        function vfToggleWidgetElements() {
            vfToggleClass('.vf-widget-toggle-button', 'vf-toggle-btn-mic');
            vfToggleClass('.vf-widget-toggle-button', 'vf-toggle-btn-close');
            vfToggleClass('.vf-widget-toggle-button', 'vf-widget-active');
            vfToggleClass('.vf-widget-toggle-button', 'vf-widget-visible');
            vfToggleClass('#vfWidgetToggleButton', 'vf-widget-float');
            vfToggleClass('.vf-widget-chat-wrapper', 'vf-widget-visible');
            vfToggleClass('.vf-widget-button', 'vf-widget-visible');
        }
    }
})();