<?php
/**
 * Plugin Name: Voice Forms
 * Description: Allows users to fill up forms using voice.
 * Version:     1.1.0
 * Author:      speak2web
 * Author URI:  https://speak2web.com/
 * Text Domain: voice-forms
 * Domain Path: /languages
 */

/**
 * Copyright (c) 2019 speak2web
 *
 * Voice Forms is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2 or, at
 * your discretion, any later version, as published by the Free
 * Software Foundation.
 *
 * Voice Forms is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Voice Forms; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

defined( 'WPINC' ) or die;

include( dirname( __FILE__ ) . '/lib/requirements-check.php' );

$voice_forms_requirements_check = new Voice_Forms_Requirements_Check( array(
    'title' => 'Voice Forms',
    'php'   => '5.3',
    'wp'    => '2.6',
    'file'  => __FILE__,
));

if ( $voice_forms_requirements_check->passes() ) {
    // Get selected language from DB and load local translation library
    $vf_selected_language = get_option( 'vf_selected_language', 'english' );
    $vf_selected_language = empty($vf_selected_language) ? 'english' : trim($vf_selected_language);
    $vf_language_file_name = strtolower($vf_selected_language) === 'german' ? 'vf_de_DE' : 'vf_en_EN';
    include( dirname( __FILE__ ) . '/classes/plugin-languages/'.$vf_language_file_name.'.php');

     try {
        switch (strtolower($vf_selected_language)) {
            case 'german':
                define('VF_LANGUAGE_LIBRARY', vf_de_DE::VF_LANGUAGE_LIB);
                break;
            default:
                define('VF_LANGUAGE_LIBRARY', vf_en_EN::VF_LANGUAGE_LIB);
        }
    } catch (\Exception $e) {
        // Do nothing for now
    }

    // Pull in the plugin classes and initialize
    include( dirname( __FILE__ ) . '/lib/wp-stack-plugin.php' );
    include( dirname( __FILE__ ) . '/classes/vf-admin-notices.php');
    include( dirname( __FILE__ ) . '/classes/plugin.php' );
    include( dirname( __FILE__ ) . '/classes/settings-page.php' );

    Voice_Forms_Plugin::start( __FILE__ );

    // Inline plugin notices
    $path = plugin_basename( __FILE__ );
    
    // Register action to hook into 'after_plugin_row_' for displaying inline notice when license is missing or invalid
    if (empty(Voice_Forms_Plugin::$vf_license_key) || empty(Voice_Forms_Plugin::$vf_api_access_key)) {
        // Go Pro notice
        add_action("after_plugin_row_{$path}", function( $plugin_file, $plugin_data, $status ) {
            echo '<tr class="active">
            <th style="border-left: 4px solid #FFB908; background-color:#FFF8E5;">&nbsp;</th>
            <td colspan="2" style="background-color:#FFF8E5;">
            <a target="blank" href="https://speak2web.com/plugin/#plan">'.VF_LANGUAGE_LIBRARY['other']['goProNotice']['goPro'].'</a>'.VF_LANGUAGE_LIBRARY['other']['goProNotice']['supportMoreBrowsers'].'
            </td>
            </tr>';
        }, 10, 3 );       
    }

    // Hook into plugin activation
    register_activation_hook(__FILE__, function() {
        $vdn_path   = 'voice-dialog-navigation/voice-dialog-navigation.php';
        $uvs_path   = 'universal-voice-search/universal-voice-search.php';
        $plugin_url = plugin_dir_url(__FILE__ );

        if (is_plugin_active($uvs_path)) {
            deactivate_plugins( $uvs_path );
        }

        if (is_plugin_active($vdn_path)) {
            wp_die(Vf_Admin_Notices::vf_denied_activation_notice($plugin_url));           
        }

        // Register plugin
        Voice_Forms_Plugin::vf_register_plugin();
    });
}

unset( $voice_forms_requirements_check );
